<?php

include_once('components/FormattingUtil.php');
include_once('data/ApiError.php');
include_once('data/OrderCallback.php');
include_once('messages/CreateOrderRequest.php');
include_once('messages/CreateOrderResponse.php');

class MerchantClient
{

	private $merchantApiUrl;
	private $secretID;
	private $vendorID;
	private $debug;

	/**
	 * @param $merchantApiUrl
	 * @param $secretID
	 * @param $vendorID
	 * @param bool $debug
	 */
	function __construct($merchantApiUrl, $secretID, $vendorID, $debug = false)
	{
		$this->merchantApiUrl = $merchantApiUrl;
		$this->secretID = $secretID;
		$this->vendorID = $vendorID;
		$this->debug = $debug;
	}
        
	public function createOrder(CreateOrderRequest $request)
	{
		$payload = array(
			'vendor' => $this->vendorID,
			'secret' => $this->secretID,
			'invoice' => $request->getOrderId(),
			'price' => $request->getPayAmount(),
			'currency' => $request->getCurrency(),
			'email' => urlencode($request->getEmail()),
			'callbackUrl' => urlencode($request->getCallbackUrl()),
			'returnUrl' => urlencode($request->getSuccessUrl())
		);
           
		if ($this->merchantApiUrl && $payload) {
			$ch=curl_init();
			curl_setopt($ch, CURLOPT_URL, $this->merchantApiUrl);
			curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
			curl_setopt($ch, CURLOPT_POST, 1);
			curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($payload));
			curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
			curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 2);
			curl_setopt($ch, CURLOPT_TIMEOUT, 100);
			$response = curl_exec($ch);
			if (curl_errno($ch)) {
				throw new \Exception('Connection Error: ' . curl_errno($ch) . ' - ' . curl_error($ch));
			}

			$httpcode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
			curl_close($ch);
			switch ($httpcode) {
				case 200:
					$data = json_decode($response);
					return new CreateOrderResponse($data->url);
					break;
				case 400:
					$data = json_decode($response);
					throw new \Exception('Error Message: <b>' . $data->message . '</b>');
				default:
					throw new \Exception('Unable to connect to Litepay.ch API. HTTP Code ->: <b>' . $httpcode. '</b>');
			}
		}
	}

	/**
	 * @param $r $_REQUEST
	 * @return OrderCallback|null
	 */
	public function parseCreateOrderCallback($r)
	{
		$result = null;

		if ($r != null && isset($r['invoice_id'], $r['secret'], $r['destination_address'], $r['input_address'], $r['value'], $r['confirmations'], $r['transaction_hash'], $r['coin'])) {
                    $result = new OrderCallback($r['invoice_id'], $r['secret'], $r['destination_address'], $r['input_address'], $r['value'], $r['confirmations'], $r['transaction_hash'], $r['coin']);
		}

		return $result;
	}


}